package tech.riemann.ims.config.auth;

import java.util.concurrent.ExecutionException;
import java.util.concurrent.TimeUnit;

import javax.annotation.Nonnull;

import org.nutz.lang.Lang;
import org.springframework.stereotype.Component;

import com.google.common.cache.CacheBuilder;
import com.google.common.cache.CacheLoader;
import com.google.common.cache.LoadingCache;

import club.zhcs.lina.auth.service.AuthUser;
import club.zhcs.lina.auth.service.UserDetailService;
import lombok.RequiredArgsConstructor;
import tech.riemann.ims.service.acl.IUserService;

/**
 * 
 */
@Component
@RequiredArgsConstructor
public class LoadingCacheUserDetailService implements UserDetailService {

    private IUserService userService;

    LoadingCache<String, AuthUser> cache = CacheBuilder.newBuilder()
                                                       .concurrencyLevel(8)
                                                       .expireAfterAccess(10, TimeUnit.MINUTES)
                                                       .initialCapacity(10)
                                                       .maximumSize(300)
                                                       .recordStats()
                                                       .build(new CacheLoader<>() {

                                                           @Override
                                                           @Nonnull
                                                           public AuthUser load(@Nonnull String key) throws Exception {
                                                               return userService.authUser(key);
                                                           }
                                                       });

    /**
     * @param name
     * @return
     * @see club.zhcs.lina.auth.service.UserDetailService#userByName(java.lang.String)
     */
    @Override
    public AuthUser userByName(String name) {
        try {
            return cache.get(name);
        }
        catch (ExecutionException e) {
            throw Lang.wrapThrow(e);
        }
    }

    /**
     * @param user
     * @see club.zhcs.lina.auth.service.UserDetailService#save(club.zhcs.lina.auth.service.AuthUser)
     */
    @Override
    public void save(AuthUser user) {
        cache.put(user.getUserName(), user);
    }

    /**
     * @param token
     * @return
     * @see club.zhcs.lina.auth.service.UserDetailService#userByToken(java.lang.String)
     */
    @Override
    public AuthUser userByToken(String token) {
        return null;
    }

}
