package tech.riemann.ims.entity.acl;

import java.io.Serial;
import java.util.Optional;

import org.nutz.dao.entity.annotation.ColDefine;
import org.nutz.dao.entity.annotation.Column;
import org.nutz.dao.entity.annotation.Comment;
import org.nutz.dao.entity.annotation.Table;
import org.nutz.json.JsonField;

import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableName;

import club.zhcs.lina.auth.service.AuthUser.Sex;
import club.zhcs.lina.utils.enums.Codebook;
import io.swagger.v3.oas.annotations.media.Schema;
import io.swagger.v3.oas.annotations.media.Schema.RequiredMode;
import lombok.AllArgsConstructor;
import lombok.Builder.Default;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.NoArgsConstructor;
import lombok.experimental.Accessors;
import lombok.experimental.FieldNameConstants;
import lombok.experimental.SuperBuilder;
import tech.riemann.ims.entity.IdBaseEntity;

/**
 * 用户
 *
 * @author Kerbores(kerbores@gmail.com)
 *
 * @since 2024-07-22 13:56:54
 */
@Data
@SuperBuilder
@NoArgsConstructor
@AllArgsConstructor
@FieldNameConstants
@EqualsAndHashCode(callSuper = true)
@Accessors(chain = true)
@TableName("t_user")
@Table("t_user")
@Comment("用户")
@Schema(name = "User", description = "用户")
public class User extends IdBaseEntity {

    @Serial
    private static final long serialVersionUID = 1L;

    @Schema(description = "用户名", requiredMode = RequiredMode.REQUIRED)
    @TableField("u_name")
    @Column("u_name")
    @Comment("用户名")
    @ColDefine(notNull = true, width = 128, precision = 0)
    private String name;

    @Schema(description = "密码", requiredMode = RequiredMode.NOT_REQUIRED)
    @TableField("u_password")
    @Column("u_password")
    @Comment("密码")
    @ColDefine(notNull = false, width = 128, precision = 0)
    private String password;

    @Schema(description = "手机号", requiredMode = RequiredMode.NOT_REQUIRED)
    @TableField("u_mobile")
    @Column("u_mobile")
    @Comment("手机号")
    @ColDefine(notNull = false, width = 128, precision = 0)
    private String mobile;

    @Schema(description = "性别", requiredMode = RequiredMode.NOT_REQUIRED)
    @TableField("u_sex")
    @Column("u_sex")
    @Comment("性别")
    @ColDefine(notNull = false, width = 20, precision = 0)
    @Default
    private Sex sex = Sex.FEMALE;

    @Schema(description = "邮箱", requiredMode = RequiredMode.NOT_REQUIRED)
    @TableField("u_email")
    @Column("u_email")
    @Comment("邮箱")
    @ColDefine(notNull = false, width = 128, precision = 0)
    private String email;

    @Schema(description = "真实姓名", requiredMode = RequiredMode.NOT_REQUIRED)
    @TableField("u_full_name")
    @Column("u_full_name")
    @Comment("真实姓名")
    @ColDefine(notNull = false, width = 128, precision = 0)
    private String fullName;

    @JsonField
    public Codebook getSexInfo() {
        return Optional.of(getSex()).orElse(Sex.FEMALE).build();
    }

    public void setSexInfo(Codebook sexInfo) {
        setSex(Sex.valueOf(sexInfo.getName()));
    }
}
